@@ -33,10 +33,13 @@ def array_array_index(array, indices):
     Returns:
         array (indices.shape): the sampled values
     """
+    if indices.shape[1] == 1:
+        return array[np.arange(array.shape[0]), indices[:, 0]].reshape(indices.shape)
+
     stride = np.arange(indices.shape[0])*array.shape[1]
     indices_mod = indices + stride[:, None]
-    indices_flat = indices_mod.flatten()
-    return array.flatten()[indices_flat].reshape(indices.shape)
+    indices_flat = indices_mod.ravel()
+    return array.ravel()[indices_flat].reshape(indices.shape).copy()
 
 def base_idx_neighbor_idx_simplices(n_base, n_neighbors=5, n_dim=2):
     """
@@ -57,6 +60,7 @@ def base_idx_neighbor_idx_simplices(n_base, n_neighbors=5, n_dim=2):
     base_indices = np.repeat(np.arange(n_base), len(combinations))
     all_simplices = np.vstack([base_indices,
                                 np.tile(combinations, (n_base, 1)).T]).T
+    #print('simplices', os.getpid(), len(all_simplices), flush=True)
     return all_simplices
 
 def all_neighbor_simplices_real_idx(n_dim, indices):
@@ -75,8 +79,12 @@ def all_neighbor_simplices_real_idx(n_dim, indices):
                                                 n_dim=n_dim)
     base_vector_indices = all_simplices[:, 0]
     neighbors_indices = indices[base_vector_indices]
+    #if debug:
+    #    print(os.getpid(), 'eee', neighbors_indices.shape, all_simplices[:,1:].shape)
     neighbors_indices = array_array_index(neighbors_indices,
                                             all_simplices[:,1:])
+    #if debug:
+    #    print(os.getpid(), 'fff')
     simplices_real_indices = np.vstack([base_vector_indices.T,
                                         neighbors_indices.T]).T
 
@@ -281,6 +289,57 @@ def add_samples(*,
     samples_by_count = [base_points + diffweight*s for s in splits]
     return np.vstack(samples_by_count)
 
+def counts_to_vector(counts):
+    """
+    Expand a count vector to a
+
+    Args:
+        counts (np.array): count vector
+
+    Returns:
+        np.array: the expanded vector
+    """
+
+    return np.hstack([np.repeat(idx, count) for idx, count in enumerate(counts)])
+
+def deterministic_sample(choices, n_to_sample, p): # pylint: disable=invalid-name
+    """
+    Take a deterministic sample
+
+    Args:
+        choices (list): the list of choices
+        n_to_sample (int): the number of samples to take
+        p (np.array): the distribution
+
+    Returns:
+        np.array: the choices
+    """
+
+    sample_counts = np.ceil(n_to_sample * p).astype(int)
+
+    n_to_remove = np.sum(sample_counts) - n_to_sample
+
+    if n_to_remove == 0:
+        return choices[counts_to_vector(sample_counts)]
+
+    non_zero_mask = sample_counts > 0
+
+    removal_indices = np.floor(np.linspace(0.0,
+                                            np.sum(non_zero_mask),
+                                            n_to_remove,
+                                            endpoint=False)).astype(int)
+
+    tmp = sample_counts[non_zero_mask]
+    tmp[removal_indices] = tmp[removal_indices] - 1
+
+    sample_counts[non_zero_mask] = tmp
+
+    assert np.sum(sample_counts) == n_to_sample
+
+    samples = choices[counts_to_vector(sample_counts)]
+
+    return samples
+
 class SimplexSamplingMixin(RandomStateMixin):
     """
     The mixin class for all simplex sampling based techniques.
@@ -288,7 +347,7 @@ class SimplexSamplingMixin(RandomStateMixin):
     def __init__(self,
                 *,
                 n_dim=2,
-                simplex_sampling='uniform',
+                simplex_sampling='random',
                 within_simplex_sampling='deterministic',
                 gaussian_component=None,
                 random_state=None):
@@ -299,7 +358,7 @@ class SimplexSamplingMixin(RandomStateMixin):
             n_dim (int): the dimensions of the simplices
                             (2 for line segments)
             simplex_sampling (str): simplex sampling method
-                                    ('uniform'/'volume'/None)
+                                    ('random'/'volume'/'deterministic'/None)
             within_simplex_sampling (str): within simplex sampling
                                     method ('random'/'deterministic')
         """
@@ -337,11 +396,11 @@ class SimplexSamplingMixin(RandomStateMixin):
         Returns:
             np.array: the distribution for sampling the simplices
         """
-        if self.simplex_sampling == 'uniform':
+        if self.simplex_sampling in ['random', 'deterministic']:
             return np.repeat(1.0/len(simplices), len(simplices))
         if self.simplex_sampling == 'volume':
             return simplex_volumes(X[simplices])
-        raise ValueError(f"simplex sampling with weighting"\
+        raise ValueError(f"simplex sampling with weighting "\
                             f"{self.simplex_sampling} not implemented yet")
 
     def all_simplices_node_weights(self, indices, simplex_weights, n_dim):
@@ -356,13 +415,21 @@ class SimplexSamplingMixin(RandomStateMixin):
         Returns:
             np.array, np.array: all simplices and the node weights
         """
+        #if 'ANS' in self.__class__.__name__:
+        #    print(os.getpid(), "zzz", indices.shape, n_dim)
+
         all_simplices = all_neighbor_simplices_real_idx(n_dim, indices)
+
+        #if 'ANS' in self.__class__.__name__:
+        #    print(os.getpid(), "ggg", indices.shape)
+
         if simplex_weights is not None:
             all_simplices_nidx = base_idx_neighbor_idx_simplices(n_base=indices.shape[0],
                                                             n_neighbors=indices.shape[1],
                                                             n_dim=n_dim)
             base_indices = all_simplices_nidx[:,0]
             neighbor_indices = all_simplices_nidx[:,1:]
+            #print(os.getpid(), "hhh", neighbor_indices.shape)
             node_weights = array_array_index(simplex_weights[base_indices], neighbor_indices)
             node_weights = np.prod(node_weights, axis=1)
         else:
@@ -425,9 +492,15 @@ class SimplexSamplingMixin(RandomStateMixin):
                                     vectors
             simplex_weights (np.array): weights of the simplices
         """
+        #if 'ANS' in self.__class__.__name__:
+        #    print(os.getpid(), "ooo")
+
         all_simplices, node_weights = \
             self.all_simplices_node_weights(indices, simplex_weights, n_dim=n_dim)
 
+        #if 'ANS' in self.__class__.__name__:
+        #    print(os.getpid(), "uuu", len(all_simplices))
+
         weights = self.determine_weights(X=X,
                                          base_weights=base_weights,
                                          X_vertices=X_vertices,
@@ -436,11 +509,24 @@ class SimplexSamplingMixin(RandomStateMixin):
 
         weights = weights * node_weights
 
-        # sample the simplices
+        #if 'ANS' in self.__class__.__name__:
+        #    print(os.getpid(), "vvv", len(weights))
+
         choices = np.arange(all_simplices.shape[0])
-        selected_indices = self.random_state.choice(choices,
+
+        if self.simplex_sampling != 'deterministic':
+            # sample the simplices
+            selected_indices = self.random_state.choice(choices,
+                                                        n_to_sample,
+                                                        p=weights/np.sum(weights))
+        elif self.simplex_sampling == 'deterministic':
+            selected_indices = deterministic_sample(choices,
                                                     n_to_sample,
                                                     p=weights/np.sum(weights))
+
+        #if 'ANS' in self.__class__.__name__:
+        #    print(os.getpid(), "www", len(selected_indices))
+
         return all_simplices[selected_indices]
 
     def add_gaussian_noise(self, samples):
