@@ -14,7 +14,7 @@ from ..base import OverSamplingSimplex
 from ..base import coalesce_dict
 
 from .._logger import logger
-_logger= logger
+_logger = logger
 
 __all__= ['A_SUWO']
 
@@ -81,7 +81,7 @@ class A_SUWO(OverSamplingSimplex):
             random_state (int/RandomState/None): initializer of random_state,
                                                     like in sklearn
         """
-        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'uniform',
+        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'random',
                                 'within_simplex_sampling': 'random',
                                 'gaussian_component': None}
 
