@@ -9,7 +9,7 @@ from ..base import OverSamplingSimplex
 from ..base import mode, coalesce_dict, coalesce
 
 from .._logger import logger
-_logger= logger
+_logger = logger
 
 __all__= ['Borderline_SMOTE1',
           'Borderline_SMOTE2']
@@ -117,7 +117,7 @@ class Borderline_SMOTE1(OverSamplingSimplex):
             random_state (int/RandomState/None): initializer of random_state,
                                                     like in sklearn
         """
-        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'uniform',
+        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'random',
                             'within_simplex_sampling': 'random',
                             'gaussian_component': None}
         ss_params = coalesce_dict(ss_params, ss_params_default)
@@ -301,7 +301,7 @@ class Borderline_SMOTE2(OverSamplingSimplex):
             random_state (int/RandomState/None): initializer of random_state,
                                                     like in sklearn
         """
-        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'uniform',
+        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'random',
                             'within_simplex_sampling': 'random',
                             'gaussian_component': None}
         ss_params = coalesce_dict(ss_params, ss_params_default)
