@@ -14,7 +14,7 @@ from ..base import RandomStateMixin, coalesce, coalesce_dict
 from ._smote import SMOTE
 
 from .._logger import logger
-_logger= logger
+_logger = logger
 
 __all__= ['DSRBF']
 
@@ -416,8 +416,8 @@ class DSRBF(OverSampling):
                  hidden_range=(4, 10),
                  i_b=2,
                  o_b=2,
-                 n_pop=500,
-                 n_init_pop=5000,
+                 n_pop=100,
+                 n_init_pop=1000,
                  n_iter=40,
                  n_sampling_epoch=5,
                  n_jobs=1,
@@ -467,7 +467,7 @@ class DSRBF(OverSampling):
         self.n_neighbors = n_neighbors
         self.nn_params = coalesce(nn_params, {})
         self.ss_params = coalesce_dict(ss_params, {'n_dim': 2,
-                                        'simplex_sampling': 'uniform',
+                                        'simplex_sampling': 'random',
                                         'within_simplex_sampling': 'random',
                                         'gaussian_component': None})
         self.params = {'m_min': hidden_range[0],
