@@ -14,7 +14,7 @@ from ..base import coalesce, coalesce_dict
 from ..base import NearestNeighborsWithMetricTensor
 from ..base import OverSamplingSimplex
 from .._logger import logger
-_logger= logger
+_logger = logger
 
 __all__= ['MOT2LD']
 
@@ -94,7 +94,7 @@ class MOT2LD(OverSamplingSimplex):
         """
         nn_params = coalesce(nn_params, {})
 
-        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'uniform',
+        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'random',
                             'within_simplex_sampling': 'random',
                             'gaussian_component': None}
         ss_params = coalesce_dict(ss_params, ss_params_default)
@@ -487,7 +487,8 @@ class MOT2LD(OverSamplingSimplex):
                         perplexity=np.min([10, X.shape[0]-1]),
                         n_iter_without_progress=100,
                         n_iter=500,
-                        verbose=0).fit_transform(X)
+                        verbose=0,
+                        n_jobs=self.n_jobs).fit_transform(X)
 
         X_min = X_tsne[y == self.min_label]
         _logger.info("%s: TSNE finished", self.__class__.__name__)
