@@ -12,7 +12,7 @@ from ..base import coalesce, coalesce_dict
 from ..base import NearestNeighborsWithMetricTensor
 from ..base import OverSamplingSimplex
 from .._logger import logger
-_logger= logger
+_logger = logger
 
 __all__= ['MWMOTE']
 
@@ -106,7 +106,7 @@ class MWMOTE(OverSamplingSimplex):
         """
         nn_params = coalesce(nn_params, {})
 
-        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'uniform',
+        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'random',
                             'within_simplex_sampling': 'random',
                             'gaussian_component': None}
         ss_params = coalesce_dict(ss_params, ss_params_default)
@@ -349,7 +349,7 @@ class MWMOTE(OverSamplingSimplex):
         Returns:
             np.array: the cluster probabilities
         """
-        cluster_weights = np.zeros(shape=(len(np.unique(cluster_labels)),))
+        cluster_weights = np.zeros(shape=(np.max(cluster_labels) + 1,))
         for idx, p_idx in enumerate(informative_minority):
             cluster_weights[cluster_labels[p_idx]] += \
                                     selection_probabilities[idx]
