@@ -8,7 +8,7 @@ from ..base import coalesce_dict
 from ..base import OverSampling, OverSamplingSimplex
 
 from .._logger import logger
-_logger= logger
+_logger = logger
 
 __all__= ['polynom_fit_SMOTE_star',
             'polynom_fit_SMOTE_bus',
@@ -440,7 +440,7 @@ class polynom_fit_SMOTE_mesh(OverSamplingSimplex): # pylint: disable=invalid-nam
             random_state (int/RandomState/None): initializer of random_state,
                                                     like in sklearn
         """
-        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'uniform',
+        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'random',
                             'within_simplex_sampling': 'deterministic',
                             'gaussian_component': None}
         ss_params = coalesce_dict(ss_params, ss_params_default)
