@@ -12,7 +12,7 @@ from ..base import OverSampling
 from ._smote import SMOTE
 
 from .._logger import logger
-_logger= logger
+_logger = logger
 
 __all__= ['SMOTE_PSOBAT']
 
@@ -103,7 +103,7 @@ class SMOTE_PSOBAT(OverSampling):
             random_state (int/RandomState/None): initializer of random_state,
                                                     like in sklearn
         """
-        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'uniform',
+        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'random',
                             'within_simplex_sampling': 'random',
                             'gaussian_component': None}
 
@@ -178,7 +178,7 @@ class SMOTE_PSOBAT(OverSampling):
         Returns:
             np.array, np.array: the predictions and the test labels
         """
-        ss_params = self.ss_params
+        ss_params = self.ss_params.copy()
         ss_params['n_dim'] = np.min([ss_params['n_dim'], int(K)])
 
         X_samp, y_samp = SMOTE(proportion=proportion,
@@ -775,7 +775,7 @@ class SMOTE_PSOBAT(OverSampling):
         elif self.method == 'bat':
             best_combination = self.bat_vectorized(X, y, nn_params)
 
-        ss_params = self.ss_params
+        ss_params = self.ss_params.copy()
         ss_params['n_dim'] = np.min([ss_params['n_dim'], int(best_combination[0])])
 
         return SMOTE(proportion=best_combination[1],
@@ -798,5 +798,6 @@ class SMOTE_PSOBAT(OverSampling):
                 'gamma': self.params['gamma'],
                 'method': self.method,
                 'nn_params': self.nn_params,
+                'ss_params': self.ss_params,
                 'n_jobs': self.n_jobs,
                 **OverSampling.get_params(self)}
