@@ -11,7 +11,7 @@ from ..base import (NearestNeighborsWithMetricTensor,
 from ..base import OverSamplingSimplex
 
 from .._logger import logger
-_logger= logger
+_logger = logger
 
 __all__= ['SOI_CJ']
 
@@ -69,7 +69,7 @@ class SOI_CJ(OverSamplingSimplex):
             random_state (int/RandomState/None): initializer of random_state,
                                                     like in sklearn
         """
-        ss_params_default = {'n_dim': 1, 'simplex_sampling': 'uniform',
+        ss_params_default = {'n_dim': 1, 'simplex_sampling': 'random',
                             'within_simplex_sampling': 'random',
                             'gaussian_component': None}
         ss_params = coalesce_dict(ss_params, ss_params_default)
