@@ -10,7 +10,7 @@ from ..base import coalesce, coalesce_dict
 from ..base import NearestNeighborsWithMetricTensor
 from ..base import OverSamplingSimplex
 from .._logger import logger
-_logger= logger
+_logger = logger
 
 __all__= ['VIS_RST']
 
@@ -74,7 +74,7 @@ class VIS_RST(OverSamplingSimplex):
             random_state (int/RandomState/None): initializer of random_state,
                                                     like in sklearn
         """
-        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'uniform',
+        ss_params_default = {'n_dim': 2, 'simplex_sampling': 'random',
                             'within_simplex_sampling': 'random',
                             'gaussian_component': None}
         ss_params = coalesce_dict(ss_params, ss_params_default)
