@@ -10,6 +10,11 @@ import smote_variants as sv
 from smote_variants.datasets import load_alternating
 from .additional_objs import additional_objs
 
+# disabling smote-variants logging
+import logging
+logger = logging.getLogger('smote_variants')
+logger.setLevel(logging.CRITICAL)
+
 dataset = load_alternating()
 
 @pytest.mark.parametrize("smote_class", sv.get_all_oversamplers())
