@@ -9,6 +9,11 @@ import smote_variants as sv
 
 from smote_variants.datasets import load_illustration_2_class
 
+# disabling smote-variants logging
+import logging
+logger = logging.getLogger('smote_variants')
+logger.setLevel(logging.CRITICAL)
+
 dataset = load_illustration_2_class()
 
 @pytest.mark.parametrize("smote_class",
@@ -21,7 +26,8 @@ def test_simplex_deterministic(smote_class):
     Args:
         smote_class (class): an oversampler class.
     """
-    ss_params = {'within_simplex_sampling': 'deterministic'}
+    ss_params = {'within_simplex_sampling': 'deterministic',
+                 'simplex_sampling': 'deterministic'}
     X, y = smote_class(ss_params=ss_params).sample(dataset['data'],
                                                    dataset['target'])
 
