@@ -9,6 +9,11 @@ import smote_variants as sv
 
 from smote_variants.datasets import load_illustration_2_class
 
+# disabling smote-variants logging
+import logging
+logger = logging.getLogger('smote_variants')
+logger.setLevel(logging.CRITICAL)
+
 dataset = load_illustration_2_class()
 
 @pytest.mark.parametrize("smote_class",
